@@ -1,6 +1,8 @@
 import numpy as np
 import pandas as pd
 
+import xarray as xr
+
 from . import parameterized
 
 
@@ -20,5 +22,5 @@ class MultiIndexSeries:
         self.series = series
 
     @parameterized(["dtype", "subset"], ([int, float], [True, False]))
-    def time_to_xarray(self, dtype, subset):
-        self.series.to_xarray()
+    def time_from_series(self, dtype, subset):
+        xr.DataArray.from_series(self.series)
