@@ -23,7 +23,7 @@ from pyproj import Transformer
 import xarray as xr
 
 # Read the data
-url = "https://github.com/mapbox/rasterio/raw/master/tests/data/RGB.byte.tif"
+url = "https://github.com/rasterio/rasterio/raw/master/tests/data/RGB.byte.tif"
 da = xr.open_rasterio(url)
 
 # Compute the lon/lat coordinates with pyproj
