@@ -18,7 +18,7 @@ import matplotlib.pyplot as plt
 import xarray as xr
 
 # Read the data
-url = "https://github.com/mapbox/rasterio/raw/master/tests/data/RGB.byte.tif"
+url = "https://github.com/rasterio/rasterio/raw/master/tests/data/RGB.byte.tif"
 da = xr.open_rasterio(url)
 
 # The data is in UTM projection. We have to set it manually until
