@@ -1,9 +1,11 @@
+from __future__ import annotations
+
 import copy
 
 import numpy as np
 
-from ..core.variable import Variable
-from .common import AbstractWritableDataStore
+from xarray.backends.common import AbstractWritableDataStore
+from xarray.core.variable import Variable
 
 
 class InMemoryDataStore(AbstractWritableDataStore):
