@@ -5,12 +5,12 @@ from collections import Counter
 import numpy as np
 import pandas as pd
 
-from .coding.times import CFDatetimeCoder, CFTimedeltaCoder
-from .conventions import decode_cf
-from .core import duck_array_ops
-from .core.dataarray import DataArray
-from .core.dtypes import get_fill_value
-from .core.pycompat import array_type
+from xarray.coding.times import CFDatetimeCoder, CFTimedeltaCoder
+from xarray.conventions import decode_cf
+from xarray.core import duck_array_ops
+from xarray.core.dataarray import DataArray
+from xarray.core.dtypes import get_fill_value
+from xarray.core.pycompat import array_type
 
 cdms2_ignored_attrs = {"name", "tileIndex"}
 iris_forbidden_keys = {
