@@ -4,12 +4,13 @@ TODO(shoyer): rewrite this module, making use of xarray.core.computation,
 NumPy's __array_ufunc__ and mixin classes instead of the unintuitive "inject"
 functions.
 """
+from __future__ import annotations
 
 import operator
 
 import numpy as np
 
-from . import dtypes, duck_array_ops
+from xarray.core import dtypes, duck_array_ops
 
 try:
     import bottleneck as bn
@@ -140,7 +141,7 @@ def fillna(data, other, join="left", dataset_join="left"):
         - "left": take only variables from the first object
         - "right": take only variables from the last object
     """
-    from .computation import apply_ufunc
+    from xarray.core.computation import apply_ufunc
 
     return apply_ufunc(
         duck_array_ops.fillna,
@@ -169,7 +170,7 @@ def where_method(self, cond, other=dtypes.NA):
     -------
     Same type as caller.
     """
-    from .computation import apply_ufunc
+    from xarray.core.computation import apply_ufunc
 
     # alignment for three arguments is complicated, so don't support it yet
     join = "inner" if other is dtypes.NA else "exact"
