@@ -16,23 +16,13 @@ from typing import (
 
 import numpy as np
 
-from .alignment import align
-from .dataarray import DataArray
-from .dataset import Dataset
-from .pycompat import is_dask_collection
-
-try:
-    import dask
-    import dask.array
-    from dask.array.utils import meta_from_array
-    from dask.highlevelgraph import HighLevelGraph
-
-except ImportError:
-    pass
-
+from xarray.core.alignment import align
+from xarray.core.dataarray import DataArray
+from xarray.core.dataset import Dataset
+from xarray.core.pycompat import is_dask_collection
 
 if TYPE_CHECKING:
-    from .types import T_Xarray
+    from xarray.core.types import T_Xarray
 
 
 def unzip(iterable):
@@ -109,6 +99,8 @@ def make_meta(obj):
     else:
         return obj
 
+    from dask.array.utils import meta_from_array
+
     meta = Dataset()
     for name, variable in obj.variables.items():
         meta_obj = meta_from_array(variable.data, ndim=variable.ndim)
@@ -165,7 +157,7 @@ def map_blocks(
     func: Callable[..., T_Xarray],
     obj: DataArray | Dataset,
     args: Sequence[Any] = (),
-    kwargs: Mapping[str, Any] = None,
+    kwargs: Mapping[str, Any] | None = None,
     template: DataArray | Dataset | None = None,
 ) -> T_Xarray:
     """Apply a function to each block of a DataArray or Dataset.
@@ -334,6 +326,14 @@ def map_blocks(
     if not is_dask_collection(obj):
         return func(obj, *args, **kwargs)
 
+    try:
+        import dask
+        import dask.array
+        from dask.highlevelgraph import HighLevelGraph
+
+    except ImportError:
+        pass
+
     all_args = [obj] + list(args)
     is_xarray = [isinstance(arg, (Dataset, DataArray)) for arg in all_args]
     is_array = [isinstance(arg, DataArray) for arg in all_args]
