@@ -1,23 +1,30 @@
+from __future__ import annotations
+
 import itertools
 import textwrap
 import warnings
 from datetime import datetime
 from inspect import getfullargspec
-from typing import Any, Iterable, Mapping, Tuple, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Hashable,
+    Iterable,
+    Mapping,
+    Sequence,
+    overload,
+)
 
 import numpy as np
 import pandas as pd
 
-from ..core.options import OPTIONS
-from ..core.pycompat import DuckArrayModule
-from ..core.utils import is_scalar
+from xarray.core.indexes import PandasMultiIndex
+from xarray.core.options import OPTIONS
+from xarray.core.pycompat import DuckArrayModule
+from xarray.core.utils import is_scalar, module_available
 
-try:
-    import nc_time_axis  # noqa: F401
-
-    nc_time_axis_available = True
-except ImportError:
-    nc_time_axis_available = False
+nc_time_axis_available = module_available("nc_time_axis")
 
 
 try:
@@ -25,8 +32,28 @@ try:
 except ImportError:
     cftime = None
 
+
+if TYPE_CHECKING:
+    from matplotlib.axes import Axes
+    from matplotlib.colors import Normalize
+    from matplotlib.ticker import FuncFormatter
+    from numpy.typing import ArrayLike
+
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
+    from xarray.core.types import AspectOptions, ScaleOptions
+
+    try:
+        import matplotlib.pyplot as plt
+    except ImportError:
+        plt: Any = None  # type: ignore
+
 ROBUST_PERCENTILE = 2.0
 
+# copied from seaborn
+_MARKERSIZE_RANGE = (18.0, 36.0, 72.0)
+_LINEWIDTH_RANGE = (1.5, 1.5, 6.0)
+
 
 def import_matplotlib_pyplot():
     """import pyplot"""
@@ -300,7 +327,12 @@ def _determine_cmap_params(
     )
 
 
-def _infer_xy_labels_3d(darray, x, y, rgb):
+def _infer_xy_labels_3d(
+    darray: DataArray | Dataset,
+    x: Hashable | None,
+    y: Hashable | None,
+    rgb: Hashable | None,
+) -> tuple[Hashable, Hashable]:
     """
     Determine x and y labels for showing RGB images.
 
@@ -354,10 +386,16 @@ def _infer_xy_labels_3d(darray, x, y, rgb):
     assert rgb is not None
 
     # Finally, we pick out the red slice and delegate to the 2D version:
-    return _infer_xy_labels(darray.isel(**{rgb: 0}), x, y)
+    return _infer_xy_labels(darray.isel({rgb: 0}), x, y)
 
 
-def _infer_xy_labels(darray, x, y, imshow=False, rgb=None):
+def _infer_xy_labels(
+    darray: DataArray | Dataset,
+    x: Hashable | None,
+    y: Hashable | None,
+    imshow: bool = False,
+    rgb: Hashable | None = None,
+) -> tuple[Hashable, Hashable]:
     """
     Determine x and y labels. For use in _plot2d
 
@@ -383,33 +421,44 @@ def _infer_xy_labels(darray, x, y, imshow=False, rgb=None):
         _assert_valid_xy(darray, x, "x")
         _assert_valid_xy(darray, y, "y")
 
-        if (
-            all(k in darray._level_coords for k in (x, y))
-            and darray._level_coords[x] == darray._level_coords[y]
-        ):
-            raise ValueError("x and y cannot be levels of the same MultiIndex")
+        if darray._indexes.get(x, 1) is darray._indexes.get(y, 2):
+            if isinstance(darray._indexes[x], PandasMultiIndex):
+                raise ValueError("x and y cannot be levels of the same MultiIndex")
 
     return x, y
 
 
-def _assert_valid_xy(darray, xy, name):
+# TODO: Can by used to more than x or y, rename?
+def _assert_valid_xy(
+    darray: DataArray | Dataset, xy: Hashable | None, name: str
+) -> None:
     """
     make sure x and y passed to plotting functions are valid
     """
 
     # MultiIndex cannot be plotted; no point in allowing them here
-    multiindex = {darray._level_coords[lc] for lc in darray._level_coords}
+    multiindex_dims = {
+        idx.dim
+        for idx in darray.xindexes.get_unique()
+        if isinstance(idx, PandasMultiIndex)
+    }
 
-    valid_xy = (
-        set(darray.dims) | set(darray.coords) | set(darray._level_coords)
-    ) - multiindex
+    valid_xy = (set(darray.dims) | set(darray.coords)) - multiindex_dims
 
-    if xy not in valid_xy:
-        valid_xy_str = "', '".join(sorted(valid_xy))
-        raise ValueError(f"{name} must be one of None, '{valid_xy_str}'")
+    if (xy is not None) and (xy not in valid_xy):
+        valid_xy_str = "', '".join(sorted(tuple(str(v) for v in valid_xy)))
+        raise ValueError(
+            f"{name} must be one of None, '{valid_xy_str}'. Received '{xy}' instead."
+        )
 
 
-def get_axis(figsize=None, size=None, aspect=None, ax=None, **kwargs):
+def get_axis(
+    figsize: Iterable[float] | None = None,
+    size: float | None = None,
+    aspect: AspectOptions = None,
+    ax: Axes | None = None,
+    **subplot_kws: Any,
+) -> Axes:
     try:
         import matplotlib as mpl
         import matplotlib.pyplot as plt
@@ -421,28 +470,36 @@ def get_axis(figsize=None, size=None, aspect=None, ax=None, **kwargs):
             raise ValueError("cannot provide both `figsize` and `ax` arguments")
         if size is not None:
             raise ValueError("cannot provide both `figsize` and `size` arguments")
-        _, ax = plt.subplots(figsize=figsize)
-    elif size is not None:
+        _, ax = plt.subplots(figsize=figsize, subplot_kw=subplot_kws)
+        return ax
+
+    if size is not None:
         if ax is not None:
             raise ValueError("cannot provide both `size` and `ax` arguments")
-        if aspect is None:
+        if aspect is None or aspect == "auto":
             width, height = mpl.rcParams["figure.figsize"]
-            aspect = width / height
-        figsize = (size * aspect, size)
-        _, ax = plt.subplots(figsize=figsize)
-    elif aspect is not None:
+            faspect = width / height
+        elif aspect == "equal":
+            faspect = 1
+        else:
+            faspect = aspect
+        figsize = (size * faspect, size)
+        _, ax = plt.subplots(figsize=figsize, subplot_kw=subplot_kws)
+        return ax
+
+    if aspect is not None:
         raise ValueError("cannot provide `aspect` argument without `size`")
 
-    if kwargs and ax is not None:
+    if subplot_kws and ax is not None:
         raise ValueError("cannot use subplot_kws with existing ax")
 
     if ax is None:
-        ax = _maybe_gca(**kwargs)
+        ax = _maybe_gca(**subplot_kws)
 
     return ax
 
 
-def _maybe_gca(**kwargs):
+def _maybe_gca(**subplot_kws: Any) -> Axes:
 
     import matplotlib.pyplot as plt
 
@@ -454,10 +511,10 @@ def _maybe_gca(**kwargs):
         # can not pass kwargs to active axes
         return plt.gca()
 
-    return plt.axes(**kwargs)
+    return plt.axes(**subplot_kws)
 
 
-def _get_units_from_attrs(da):
+def _get_units_from_attrs(da) -> str:
     """Extracts and formats the unit/units from a attributes."""
     pint_array_type = DuckArrayModule("pint").type
     units = " [{}]"
@@ -472,16 +529,16 @@ def _get_units_from_attrs(da):
     return units
 
 
-def label_from_attrs(da, extra=""):
+def label_from_attrs(da, extra: str = "") -> str:
     """Makes informative labels if variable metadata (attrs) follows
     CF conventions."""
-
+    name: str = "{}"
     if da.attrs.get("long_name"):
-        name = da.attrs["long_name"]
+        name = name.format(da.attrs["long_name"])
     elif da.attrs.get("standard_name"):
-        name = da.attrs["standard_name"]
+        name = name.format(da.attrs["standard_name"])
     elif da.name is not None:
-        name = da.name
+        name = name.format(da.name)
     else:
         name = ""
 
@@ -496,7 +553,7 @@ def label_from_attrs(da, extra=""):
         return "\n".join(textwrap.wrap(name + extra + units, 30))
 
 
-def _interval_to_mid_points(array):
+def _interval_to_mid_points(array: Iterable[pd.Interval]) -> np.ndarray:
     """
     Helper function which returns an array
     with the Intervals' mid points.
@@ -505,7 +562,7 @@ def _interval_to_mid_points(array):
     return np.array([x.mid for x in array])
 
 
-def _interval_to_bound_points(array):
+def _interval_to_bound_points(array: Sequence[pd.Interval]) -> np.ndarray:
     """
     Helper function which returns an array
     with the Intervals' boundaries.
@@ -517,7 +574,9 @@ def _interval_to_bound_points(array):
     return array_boundaries
 
 
-def _interval_to_double_bound_points(xarray, yarray):
+def _interval_to_double_bound_points(
+    xarray: Iterable[pd.Interval], yarray: Iterable
+) -> tuple[np.ndarray, np.ndarray]:
     """
     Helper function to deal with a xarray consisting of pd.Intervals. Each
     interval is replaced with both boundaries. I.e. the length of xarray
@@ -527,13 +586,15 @@ def _interval_to_double_bound_points(xarray, yarray):
     xarray1 = np.array([x.left for x in xarray])
     xarray2 = np.array([x.right for x in xarray])
 
-    xarray = list(itertools.chain.from_iterable(zip(xarray1, xarray2)))
-    yarray = list(itertools.chain.from_iterable(zip(yarray, yarray)))
+    xarray_out = np.array(list(itertools.chain.from_iterable(zip(xarray1, xarray2))))
+    yarray_out = np.array(list(itertools.chain.from_iterable(zip(yarray, yarray))))
 
-    return xarray, yarray
+    return xarray_out, yarray_out
 
 
-def _resolve_intervals_1dplot(xval, yval, kwargs):
+def _resolve_intervals_1dplot(
+    xval: np.ndarray, yval: np.ndarray, kwargs: dict
+) -> tuple[np.ndarray, np.ndarray, str, str, dict]:
     """
     Helper function to replace the values of x and/or y coordinate arrays
     containing pd.Interval with their mid-points or - for step plots - double
@@ -546,13 +607,16 @@ def _resolve_intervals_1dplot(xval, yval, kwargs):
     if kwargs.get("drawstyle", "").startswith("steps-"):
 
         remove_drawstyle = False
+
         # Convert intervals to double points
-        if _valid_other_type(np.array([xval, yval]), [pd.Interval]):
+        x_is_interval = _valid_other_type(xval, pd.Interval)
+        y_is_interval = _valid_other_type(yval, pd.Interval)
+        if x_is_interval and y_is_interval:
             raise TypeError("Can't step plot intervals against intervals.")
-        if _valid_other_type(xval, [pd.Interval]):
+        elif x_is_interval:
             xval, yval = _interval_to_double_bound_points(xval, yval)
             remove_drawstyle = True
-        if _valid_other_type(yval, [pd.Interval]):
+        elif y_is_interval:
             yval, xval = _interval_to_double_bound_points(yval, xval)
             remove_drawstyle = True
 
@@ -564,10 +628,10 @@ def _resolve_intervals_1dplot(xval, yval, kwargs):
     else:
 
         # Convert intervals to mid points and adjust labels
-        if _valid_other_type(xval, [pd.Interval]):
+        if _valid_other_type(xval, pd.Interval):
             xval = _interval_to_mid_points(xval)
             x_suffix = "_center"
-        if _valid_other_type(yval, [pd.Interval]):
+        if _valid_other_type(yval, pd.Interval):
             yval = _interval_to_mid_points(yval)
             y_suffix = "_center"
 
@@ -582,7 +646,7 @@ def _resolve_intervals_2dplot(val, func_name):
     increases length by 1.
     """
     label_extra = ""
-    if _valid_other_type(val, [pd.Interval]):
+    if _valid_other_type(val, pd.Interval):
         if func_name == "pcolormesh":
             val = _interval_to_bound_points(val)
         else:
@@ -592,11 +656,13 @@ def _resolve_intervals_2dplot(val, func_name):
     return val, label_extra
 
 
-def _valid_other_type(x, types):
+def _valid_other_type(
+    x: ArrayLike, types: type[object] | tuple[type[object], ...]
+) -> bool:
     """
     Do all elements of x have a type from types?
     """
-    return all(any(isinstance(el, t) for t in types) for el in np.ravel(x))
+    return all(isinstance(el, types) for el in np.ravel(x))
 
 
 def _valid_numpy_subdtype(x, numpy_types):
@@ -611,47 +677,49 @@ def _valid_numpy_subdtype(x, numpy_types):
     return any(np.issubdtype(x.dtype, t) for t in numpy_types)
 
 
-def _ensure_plottable(*args):
+def _ensure_plottable(*args) -> None:
     """
     Raise exception if there is anything in args that can't be plotted on an
     axis by matplotlib.
     """
-    numpy_types = [
+    numpy_types: tuple[type[object], ...] = (
         np.floating,
         np.integer,
         np.timedelta64,
         np.datetime64,
         np.bool_,
         np.str_,
-    ]
-    other_types = [datetime]
-    if cftime is not None:
-        cftime_datetime_types = [cftime.datetime]
-        other_types = other_types + cftime_datetime_types
-    else:
-        cftime_datetime_types = []
+    )
+    other_types: tuple[type[object], ...] = (datetime,)
+    cftime_datetime_types: tuple[type[object], ...] = (
+        () if cftime is None else (cftime.datetime,)
+    )
+    other_types += cftime_datetime_types
+
     for x in args:
         if not (
-            _valid_numpy_subdtype(np.array(x), numpy_types)
-            or _valid_other_type(np.array(x), other_types)
+            _valid_numpy_subdtype(np.asarray(x), numpy_types)
+            or _valid_other_type(np.asarray(x), other_types)
         ):
             raise TypeError(
                 "Plotting requires coordinates to be numeric, boolean, "
                 "or dates of type numpy.datetime64, "
                 "datetime.datetime, cftime.datetime or "
-                f"pandas.Interval. Received data of type {np.array(x).dtype} instead."
-            )
-        if (
-            _valid_other_type(np.array(x), cftime_datetime_types)
-            and not nc_time_axis_available
-        ):
-            raise ImportError(
-                "Plotting of arrays of cftime.datetime "
-                "objects or arrays indexed by "
-                "cftime.datetime objects requires the "
-                "optional `nc-time-axis` (v1.2.0 or later) "
-                "package."
+                f"pandas.Interval. Received data of type {np.asarray(x).dtype} instead."
             )
+        if _valid_other_type(np.asarray(x), cftime_datetime_types):
+            if nc_time_axis_available:
+                # Register cftime datetypes to matplotlib.units.registry,
+                # otherwise matplotlib will raise an error:
+                import nc_time_axis  # noqa: F401
+            else:
+                raise ImportError(
+                    "Plotting of arrays of cftime.datetime "
+                    "objects or arrays indexed by "
+                    "cftime.datetime objects requires the "
+                    "optional `nc-time-axis` (v1.2.0 or later) "
+                    "package."
+                )
 
 
 def _is_numeric(arr):
@@ -711,16 +779,16 @@ def _rescale_imshow_rgb(darray, vmin, vmax, robust):
 
 
 def _update_axes(
-    ax,
-    xincrease,
-    yincrease,
-    xscale=None,
-    yscale=None,
-    xticks=None,
-    yticks=None,
-    xlim=None,
-    ylim=None,
-):
+    ax: Axes,
+    xincrease: bool | None,
+    yincrease: bool | None,
+    xscale: ScaleOptions = None,
+    yscale: ScaleOptions = None,
+    xticks: ArrayLike | None = None,
+    yticks: ArrayLike | None = None,
+    xlim: ArrayLike | None = None,
+    ylim: ArrayLike | None = None,
+) -> None:
     """
     Update axes with provided parameters
     """
@@ -835,11 +903,11 @@ def _process_cmap_cbar_kwargs(
     data,
     cmap=None,
     colors=None,
-    cbar_kwargs: Union[Iterable[Tuple[str, Any]], Mapping[str, Any]] = None,
+    cbar_kwargs: Iterable[tuple[str, Any]] | Mapping[str, Any] | None = None,
     levels=None,
     _is_facetgrid=False,
     **kwargs,
-):
+) -> tuple[dict[str, Any], dict[str, Any]]:
     """
     Parameters
     ----------
@@ -849,8 +917,8 @@ def _process_cmap_cbar_kwargs(
 
     Returns
     -------
-    cmap_params
-    cbar_kwargs
+    cmap_params : dict
+    cbar_kwargs : dict
     """
     if func.__name__ == "surface":
         # Leave user to specify cmap settings for surface plots
@@ -1002,7 +1070,10 @@ def legend_elements(
             return self.cmap(self.norm(value)), _size
 
     elif prop == "sizes":
-        arr = self.get_sizes()
+        if isinstance(self, mpl.collections.LineCollection):
+            arr = self.get_linewidths()
+        else:
+            arr = self.get_sizes()
         _color = kwargs.pop("color", "k")
 
         def _get_color_and_size(value):
@@ -1036,7 +1107,8 @@ def legend_elements(
     if label_values_are_numeric:
         label_values_min = label_values.min()
         label_values_max = label_values.max()
-        fmt.set_bounds(label_values_min, label_values_max)
+        fmt.axis.set_view_interval(label_values_min, label_values_max)
+        fmt.axis.set_data_interval(label_values_min, label_values_max)
 
         if num is not None:
             # Labels are numerical but larger than the target
@@ -1090,22 +1162,29 @@ def legend_elements(
 
     for val, lab in zip(values, label_values):
         color, size = _get_color_and_size(val)
-        h = mlines.Line2D(
-            [0], [0], ls="", color=color, ms=size, marker=self.get_paths()[0], **kw
-        )
+
+        if isinstance(self, mpl.collections.PathCollection):
+            kw.update(linestyle="", marker=self.get_paths()[0], markersize=size)
+        elif isinstance(self, mpl.collections.LineCollection):
+            kw.update(linestyle=self.get_linestyle()[0], linewidth=size)
+
+        h = mlines.Line2D([0], [0], color=color, **kw)
+
         handles.append(h)
         labels.append(fmt(lab))
 
     return handles, labels
 
 
-def _legend_add_subtitle(handles, labels, text, func):
+def _legend_add_subtitle(handles, labels, text):
     """Add a subtitle to legend handles."""
+    plt = import_matplotlib_pyplot()
+
     if text and len(handles) > 1:
         # Create a blank handle that's not visible, the
         # invisibillity will be used to discern which are subtitles
         # or not:
-        blank_handle = func([], [], label=text)
+        blank_handle = plt.Line2D([], [], label=text)
         blank_handle.set_visible(False)
 
         # Subtitles are shown first:
@@ -1122,8 +1201,13 @@ def _adjust_legend_subtitles(legend):
     # Legend title not in rcParams until 3.0
     font_size = plt.rcParams.get("legend.title_fontsize", None)
     hpackers = legend.findobj(plt.matplotlib.offsetbox.VPacker)[0].get_children()
+    hpackers = [v for v in hpackers if isinstance(v, plt.matplotlib.offsetbox.HPacker)]
     for hpack in hpackers:
-        draw_area, text_area = hpack.get_children()
+        areas = hpack.get_children()
+        if len(areas) < 2:
+            continue
+        draw_area, text_area = areas
+
         handles = draw_area.get_children()
 
         # Assume that all artists that are not visible are
@@ -1137,3 +1221,531 @@ def _adjust_legend_subtitles(legend):
                     # The sutbtitles should have the same font size
                     # as normal legend titles:
                     text.set_size(font_size)
+
+
+def _infer_meta_data(ds, x, y, hue, hue_style, add_guide, funcname):
+    dvars = set(ds.variables.keys())
+    error_msg = f" must be one of ({', '.join(sorted(tuple(str(v) for v in dvars)))})"
+
+    if x not in dvars:
+        raise ValueError(f"Expected 'x' {error_msg}. Received {x} instead.")
+
+    if y not in dvars:
+        raise ValueError(f"Expected 'y' {error_msg}. Received {y} instead.")
+
+    if hue is not None and hue not in dvars:
+        raise ValueError(f"Expected 'hue' {error_msg}. Received {hue} instead.")
+
+    if hue:
+        hue_is_numeric = _is_numeric(ds[hue].values)
+
+        if hue_style is None:
+            hue_style = "continuous" if hue_is_numeric else "discrete"
+
+        if not hue_is_numeric and (hue_style == "continuous"):
+            raise ValueError(
+                f"Cannot create a colorbar for a non numeric coordinate: {hue}"
+            )
+
+        if add_guide is None or add_guide is True:
+            add_colorbar = True if hue_style == "continuous" else False
+            add_legend = True if hue_style == "discrete" else False
+        else:
+            add_colorbar = False
+            add_legend = False
+    else:
+        if add_guide is True and funcname not in ("quiver", "streamplot"):
+            raise ValueError("Cannot set add_guide when hue is None.")
+        add_legend = False
+        add_colorbar = False
+
+    if (add_guide or add_guide is None) and funcname == "quiver":
+        add_quiverkey = True
+        if hue:
+            add_colorbar = True
+            if not hue_style:
+                hue_style = "continuous"
+            elif hue_style != "continuous":
+                raise ValueError(
+                    "hue_style must be 'continuous' or None for .plot.quiver or "
+                    ".plot.streamplot"
+                )
+    else:
+        add_quiverkey = False
+
+    if (add_guide or add_guide is None) and funcname == "streamplot":
+        if hue:
+            add_colorbar = True
+            if not hue_style:
+                hue_style = "continuous"
+            elif hue_style != "continuous":
+                raise ValueError(
+                    "hue_style must be 'continuous' or None for .plot.quiver or "
+                    ".plot.streamplot"
+                )
+
+    if hue_style is not None and hue_style not in ["discrete", "continuous"]:
+        raise ValueError("hue_style must be either None, 'discrete' or 'continuous'.")
+
+    if hue:
+        hue_label = label_from_attrs(ds[hue])
+        hue = ds[hue]
+    else:
+        hue_label = None
+        hue = None
+
+    return {
+        "add_colorbar": add_colorbar,
+        "add_legend": add_legend,
+        "add_quiverkey": add_quiverkey,
+        "hue_label": hue_label,
+        "hue_style": hue_style,
+        "xlabel": label_from_attrs(ds[x]),
+        "ylabel": label_from_attrs(ds[y]),
+        "hue": hue,
+    }
+
+
+@overload
+def _parse_size(
+    data: None,
+    norm: tuple[float | None, float | None, bool] | Normalize | None,
+) -> None:
+    ...
+
+
+@overload
+def _parse_size(
+    data: DataArray,
+    norm: tuple[float | None, float | None, bool] | Normalize | None,
+) -> pd.Series:
+    ...
+
+
+# copied from seaborn
+def _parse_size(
+    data: DataArray | None,
+    norm: tuple[float | None, float | None, bool] | Normalize | None,
+) -> None | pd.Series:
+
+    import matplotlib as mpl
+
+    if data is None:
+        return None
+
+    flatdata = data.values.flatten()
+
+    if not _is_numeric(flatdata):
+        levels = np.unique(flatdata)
+        numbers = np.arange(1, 1 + len(levels))[::-1]
+    else:
+        levels = numbers = np.sort(np.unique(flatdata))
+
+    min_width, default_width, max_width = _MARKERSIZE_RANGE
+    # width_range = min_width, max_width
+
+    if norm is None:
+        norm = mpl.colors.Normalize()
+    elif isinstance(norm, tuple):
+        norm = mpl.colors.Normalize(*norm)
+    elif not isinstance(norm, mpl.colors.Normalize):
+        err = "``size_norm`` must be None, tuple, or Normalize object."
+        raise ValueError(err)
+    assert isinstance(norm, mpl.colors.Normalize)
+
+    norm.clip = True
+    if not norm.scaled():
+        norm(np.asarray(numbers))
+    # limits = norm.vmin, norm.vmax
+
+    scl = norm(numbers)
+    widths = np.asarray(min_width + scl * (max_width - min_width))
+    if scl.mask.any():
+        widths[scl.mask] = 0
+    sizes = dict(zip(levels, widths))
+
+    return pd.Series(sizes)
+
+
+class _Normalize(Sequence):
+    """
+    Normalize numerical or categorical values to numerical values.
+
+    The class includes helper methods that simplifies transforming to
+    and from normalized values.
+
+    Parameters
+    ----------
+    data : DataArray
+        DataArray to normalize.
+    width : Sequence of three numbers, optional
+        Normalize the data to these (min, default, max) values.
+        The default is None.
+    """
+
+    _data: DataArray | None
+    _data_unique: np.ndarray
+    _data_unique_index: np.ndarray
+    _data_unique_inverse: np.ndarray
+    _data_is_numeric: bool
+    _width: tuple[float, float, float] | None
+
+    __slots__ = (
+        "_data",
+        "_data_unique",
+        "_data_unique_index",
+        "_data_unique_inverse",
+        "_data_is_numeric",
+        "_width",
+    )
+
+    def __init__(
+        self,
+        data: DataArray | None,
+        width: tuple[float, float, float] | None = None,
+        _is_facetgrid: bool = False,
+    ) -> None:
+        self._data = data
+        self._width = width if not _is_facetgrid else None
+
+        pint_array_type = DuckArrayModule("pint").type
+        to_unique = (
+            data.to_numpy()  # type: ignore[union-attr]
+            if isinstance(data if data is None else data.data, pint_array_type)
+            else data
+        )
+        data_unique, data_unique_inverse = np.unique(to_unique, return_inverse=True)  # type: ignore[call-overload]
+        self._data_unique = data_unique
+        self._data_unique_index = np.arange(0, data_unique.size)
+        self._data_unique_inverse = data_unique_inverse
+        self._data_is_numeric = False if data is None else _is_numeric(data)
+
+    def __repr__(self) -> str:
+        with np.printoptions(precision=4, suppress=True, threshold=5):
+            return (
+                f"<_Normalize(data, width={self._width})>\n"
+                f"{self._data_unique} -> {self._values_unique}"
+            )
+
+    def __len__(self) -> int:
+        return len(self._data_unique)
+
+    def __getitem__(self, key):
+        return self._data_unique[key]
+
+    @property
+    def data(self) -> DataArray | None:
+        return self._data
+
+    @property
+    def data_is_numeric(self) -> bool:
+        """
+        Check if data is numeric.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).data_is_numeric
+        False
+
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> _Normalize(a).data_is_numeric
+        True
+        """
+        return self._data_is_numeric
+
+    @overload
+    def _calc_widths(self, y: np.ndarray) -> np.ndarray:
+        ...
+
+    @overload
+    def _calc_widths(self, y: DataArray) -> DataArray:
+        ...
+
+    def _calc_widths(self, y: np.ndarray | DataArray) -> np.ndarray | DataArray:
+        """
+        Normalize the values so they're inbetween self._width.
+        """
+        if self._width is None:
+            return y
+
+        xmin, xdefault, xmax = self._width
+
+        diff_maxy_miny = np.max(y) - np.min(y)
+        if diff_maxy_miny == 0:
+            # Use default with if y is constant:
+            widths = xdefault + 0 * y
+        else:
+            # Normalize inbetween xmin and xmax:
+            k = (y - np.min(y)) / diff_maxy_miny
+            widths = xmin + k * (xmax - xmin)
+        return widths
+
+    @overload
+    def _indexes_centered(self, x: np.ndarray) -> np.ndarray:
+        ...
+
+    @overload
+    def _indexes_centered(self, x: DataArray) -> DataArray:
+        ...
+
+    def _indexes_centered(self, x: np.ndarray | DataArray) -> np.ndarray | DataArray:
+        """
+        Offset indexes to make sure being in the center of self.levels.
+        ["a", "b", "c"] -> [1, 3, 5]
+        """
+        return x * 2 + 1
+
+    @property
+    def values(self) -> DataArray | None:
+        """
+        Return a normalized number array for the unique levels.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).values
+        <xarray.DataArray (dim_0: 5)>
+        array([3, 1, 1, 3, 5])
+        Dimensions without coordinates: dim_0
+
+        >>> _Normalize(a, width=(18, 36, 72)).values
+        <xarray.DataArray (dim_0: 5)>
+        array([45., 18., 18., 45., 72.])
+        Dimensions without coordinates: dim_0
+
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> _Normalize(a).values
+        <xarray.DataArray (dim_0: 6)>
+        array([0.5, 0. , 0. , 0.5, 2. , 3. ])
+        Dimensions without coordinates: dim_0
+
+        >>> _Normalize(a, width=(18, 36, 72)).values
+        <xarray.DataArray (dim_0: 6)>
+        array([27., 18., 18., 27., 54., 72.])
+        Dimensions without coordinates: dim_0
+
+        >>> _Normalize(a * 0, width=(18, 36, 72)).values
+        <xarray.DataArray (dim_0: 6)>
+        array([36., 36., 36., 36., 36., 36.])
+        Dimensions without coordinates: dim_0
+
+        """
+        if self.data is None:
+            return None
+
+        val: DataArray
+        if self.data_is_numeric:
+            val = self.data
+        else:
+            arr = self._indexes_centered(self._data_unique_inverse)
+            val = self.data.copy(data=arr.reshape(self.data.shape))
+
+        return self._calc_widths(val)
+
+    @property
+    def _values_unique(self) -> np.ndarray | None:
+        """
+        Return unique values.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a)._values_unique
+        array([1, 3, 5])
+
+        >>> _Normalize(a, width=(18, 36, 72))._values_unique
+        array([18., 45., 72.])
+
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> _Normalize(a)._values_unique
+        array([0. , 0.5, 2. , 3. ])
+
+        >>> _Normalize(a, width=(18, 36, 72))._values_unique
+        array([18., 27., 54., 72.])
+        """
+        if self.data is None:
+            return None
+
+        val: np.ndarray
+        if self.data_is_numeric:
+            val = self._data_unique
+        else:
+            val = self._indexes_centered(self._data_unique_index)
+
+        return self._calc_widths(val)
+
+    @property
+    def ticks(self) -> np.ndarray | None:
+        """
+        Return ticks for plt.colorbar if the data is not numeric.
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).ticks
+        array([1, 3, 5])
+        """
+        val: None | np.ndarray
+        if self.data_is_numeric:
+            val = None
+        else:
+            val = self._indexes_centered(self._data_unique_index)
+
+        return val
+
+    @property
+    def levels(self) -> np.ndarray:
+        """
+        Return discrete levels that will evenly bound self.values.
+        ["a", "b", "c"] -> [0, 2, 4, 6]
+
+        Examples
+        --------
+        >>> a = xr.DataArray(["b", "a", "a", "b", "c"])
+        >>> _Normalize(a).levels
+        array([0, 2, 4, 6])
+        """
+        return (
+            np.append(self._data_unique_index, np.max(self._data_unique_index) + 1) * 2
+        )
+
+    @property
+    def _lookup(self) -> pd.Series:
+        if self._values_unique is None:
+            raise ValueError("self.data can't be None.")
+
+        return pd.Series(dict(zip(self._values_unique, self._data_unique)))
+
+    def _lookup_arr(self, x) -> np.ndarray:
+        # Use reindex to be less sensitive to float errors. reindex only
+        # works with sorted index.
+        # Return as numpy array since legend_elements
+        # seems to require that:
+        return self._lookup.sort_index().reindex(x, method="nearest").to_numpy()
+
+    @property
+    def format(self) -> FuncFormatter:
+        """
+        Return a FuncFormatter that maps self.values elements back to
+        the original value as a string. Useful with plt.colorbar.
+
+        Examples
+        --------
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> aa = _Normalize(a, width=(0, 0.5, 1))
+        >>> aa._lookup
+        0.000000    0.0
+        0.166667    0.5
+        0.666667    2.0
+        1.000000    3.0
+        dtype: float64
+        >>> aa.format(1)
+        '3.0'
+        """
+        plt = import_matplotlib_pyplot()
+
+        def _func(x: Any, pos: None | Any = None):
+            return f"{self._lookup_arr([x])[0]}"
+
+        return plt.FuncFormatter(_func)
+
+    @property
+    def func(self) -> Callable[[Any, None | Any], Any]:
+        """
+        Return a lambda function that maps self.values elements back to
+        the original value as a numpy array. Useful with ax.legend_elements.
+
+        Examples
+        --------
+        >>> a = xr.DataArray([0.5, 0, 0, 0.5, 2, 3])
+        >>> aa = _Normalize(a, width=(0, 0.5, 1))
+        >>> aa._lookup
+        0.000000    0.0
+        0.166667    0.5
+        0.666667    2.0
+        1.000000    3.0
+        dtype: float64
+        >>> aa.func([0.16, 1])
+        array([0.5, 3. ])
+        """
+
+        def _func(x: Any, pos: None | Any = None):
+            return self._lookup_arr(x)
+
+        return _func
+
+
+def _determine_guide(
+    hueplt_norm: _Normalize,
+    sizeplt_norm: _Normalize,
+    add_colorbar: None | bool = None,
+    add_legend: None | bool = None,
+    plotfunc_name: str | None = None,
+) -> tuple[bool, bool]:
+    if plotfunc_name == "hist":
+        return False, False
+
+    if (add_colorbar) and hueplt_norm.data is None:
+        raise KeyError("Cannot create a colorbar when hue is None.")
+    if add_colorbar is None:
+        if hueplt_norm.data is not None:
+            add_colorbar = True
+        else:
+            add_colorbar = False
+
+    if add_legend and hueplt_norm.data is None and sizeplt_norm.data is None:
+        raise KeyError("Cannot create a legend when hue and markersize is None.")
+    if add_legend is None:
+        if (
+            not add_colorbar
+            and (hueplt_norm.data is not None and hueplt_norm.data_is_numeric is False)
+            or sizeplt_norm.data is not None
+        ):
+            add_legend = True
+        else:
+            add_legend = False
+
+    return add_colorbar, add_legend
+
+
+def _add_legend(
+    hueplt_norm: _Normalize,
+    sizeplt_norm: _Normalize,
+    primitive,
+    legend_ax,
+    plotfunc: str,
+):
+
+    primitive = primitive if isinstance(primitive, list) else [primitive]
+
+    handles, labels = [], []
+    for huesizeplt, prop in [
+        (hueplt_norm, "colors"),
+        (sizeplt_norm, "sizes"),
+    ]:
+        if huesizeplt.data is not None:
+            # Get legend handles and labels that displays the
+            # values correctly. Order might be different because
+            # legend_elements uses np.unique instead of pd.unique,
+            # FacetGrid.add_legend might have troubles with this:
+            hdl, lbl = [], []
+            for p in primitive:
+                hdl_, lbl_ = legend_elements(p, prop, num="auto", func=huesizeplt.func)
+                hdl += hdl_
+                lbl += lbl_
+
+            # Only save unique values:
+            u, ind = np.unique(lbl, return_index=True)
+            ind = np.argsort(ind)
+            lbl = u[ind].tolist()
+            hdl = np.array(hdl)[ind].tolist()
+
+            # Add a subtitle:
+            hdl, lbl = _legend_add_subtitle(hdl, lbl, label_from_attrs(huesizeplt.data))
+            handles += hdl
+            labels += lbl
+    legend = legend_ax.legend(handles, labels, framealpha=0.5)
+    _adjust_legend_subtitles(legend)
+
+    return legend
