@@ -10,8 +10,7 @@ from numpy.core import defchararray
 
 import xarray as xr
 from xarray.core import formatting
-
-from . import requires_dask, requires_netCDF4
+from xarray.tests import requires_dask, requires_netCDF4
 
 
 class TestFormatting:
@@ -219,6 +218,31 @@ class TestFormatting:
         assert "\n" not in newlines
         assert "\t" not in tabs
 
+    def test_index_repr(self):
+        from xarray.core.indexes import Index
+
+        class CustomIndex(Index):
+            def __init__(self, names):
+                self.names = names
+
+            def __repr__(self):
+                return f"CustomIndex(coords={self.names})"
+
+        coord_names = ["x", "y"]
+        index = CustomIndex(coord_names)
+        name = "x"
+
+        normal = formatting.summarize_index(name, index, col_width=20)
+        assert name in normal
+        assert "CustomIndex" in normal
+
+        CustomIndex._repr_inline_ = (
+            lambda self, max_width: f"CustomIndex[{', '.join(self.names)}]"
+        )
+        inline = formatting.summarize_index(name, index, col_width=20)
+        assert name in inline
+        assert index._repr_inline_(max_width=40) in inline
+
     def test_diff_array_repr(self) -> None:
         da_a = xr.DataArray(
             np.array([[1, 2, 3], [4, 5, 6]], dtype="int64"),
@@ -391,7 +415,10 @@ class TestFormatting:
     def test_array_repr(self) -> None:
         ds = xr.Dataset(coords={"foo": [1, 2, 3], "bar": [1, 2, 3]})
         ds[(1, 2)] = xr.DataArray([0], dims="test")
-        actual = formatting.array_repr(ds[(1, 2)])
+        ds_12 = ds[(1, 2)]
+
+        # Test repr function behaves correctly:
+        actual = formatting.array_repr(ds_12)
         expected = dedent(
             """\
         <xarray.DataArray (1, 2) (test: 1)>
@@ -401,6 +428,14 @@ class TestFormatting:
 
         assert actual == expected
 
+        # Test repr, str prints returns correctly as well:
+        assert repr(ds_12) == expected
+        assert str(ds_12) == expected
+
+        # f-strings (aka format(...)) by default should use the repr:
+        actual = f"{ds_12}"
+        assert actual == expected
+
         with xr.set_options(display_expand_data=False):
             actual = formatting.array_repr(ds[(1, 2)])
             expected = dedent(
@@ -420,25 +455,46 @@ class TestFormatting:
         with xr.set_options(display_expand_data=False):
             formatting.array_repr(var)
 
+    def test_array_repr_recursive(self) -> None:
+        # GH:issue:7111
+
+        # direct recurion
+        var = xr.Variable("x", [0, 1])
+        var.attrs["x"] = var
+        formatting.array_repr(var)
+
+        da = xr.DataArray([0, 1], dims=["x"])
+        da.attrs["x"] = da
+        formatting.array_repr(da)
+
+        # indirect recursion
+        var.attrs["x"] = da
+        da.attrs["x"] = var
+        formatting.array_repr(var)
+        formatting.array_repr(da)
+
     @requires_dask
     def test_array_scalar_format(self) -> None:
-        var = xr.DataArray(0)
-        assert var.__format__("") == "0"
-        assert var.__format__("d") == "0"
-        assert var.__format__(".2f") == "0.00"
+        # Test numpy scalars:
+        var = xr.DataArray(np.array(0))
+        assert format(var, "") == repr(var)
+        assert format(var, "d") == "0"
+        assert format(var, ".2f") == "0.00"
 
+        # Test dask scalars, not supported however:
+        import dask.array as da
+
+        var = xr.DataArray(da.array(0))
+        assert format(var, "") == repr(var)
+        with pytest.raises(TypeError) as excinfo:
+            format(var, ".2f")
+        assert "unsupported format string passed to" in str(excinfo.value)
+
+        # Test numpy arrays raises:
         var = xr.DataArray([0.1, 0.2])
-        assert var.__format__("") == "[0.1 0.2]"
-        with pytest.raises(TypeError) as excinfo:
-            var.__format__(".2f")
-        assert "unsupported format string passed to" in str(excinfo.value)
-
-        # also check for dask
-        var = var.chunk(chunks={"dim_0": 1})
-        assert var.__format__("") == "[0.1 0.2]"
-        with pytest.raises(TypeError) as excinfo:
-            var.__format__(".2f")
-        assert "unsupported format string passed to" in str(excinfo.value)
+        with pytest.raises(NotImplementedError) as excinfo:  # type: ignore
+            format(var, ".2f")
+        assert "Using format_spec is only supported" in str(excinfo.value)
 
 
 def test_inline_variable_array_repr_custom_repr() -> None:
@@ -518,17 +574,28 @@ def test_large_array_repr_length() -> None:
 
 @requires_netCDF4
 def test_repr_file_collapsed(tmp_path) -> None:
-    arr = xr.DataArray(np.arange(300), dims="test")
-    arr.to_netcdf(tmp_path / "test.nc", engine="netcdf4")
+    arr_to_store = xr.DataArray(np.arange(300, dtype=np.int64), dims="test")
+    arr_to_store.to_netcdf(tmp_path / "test.nc", engine="netcdf4")
 
     with xr.open_dataarray(tmp_path / "test.nc") as arr, xr.set_options(
         display_expand_data=False
     ):
-        actual = formatting.array_repr(arr)
+        actual = repr(arr)
         expected = dedent(
             """\
         <xarray.DataArray (test: 300)>
-        array([  0,   1,   2, ..., 297, 298, 299])
+        [300 values with dtype=int64]
+        Dimensions without coordinates: test"""
+        )
+
+        assert actual == expected
+
+        arr_loaded = arr.compute()
+        actual = arr_loaded.__repr__()
+        expected = dedent(
+            """\
+        <xarray.DataArray (test: 300)>
+        0 1 2 3 4 5 6 7 8 9 10 11 12 ... 288 289 290 291 292 293 294 295 296 297 298 299
         Dimensions without coordinates: test"""
         )
 
@@ -601,6 +668,21 @@ Attributes: ({n_attr})"""
         assert actual == expected
 
 
+def test__mapping_repr_recursive() -> None:
+    # GH:issue:7111
+
+    # direct recursion
+    ds = xr.Dataset({"a": ("x", [1, 2, 3])})
+    ds.attrs["ds"] = ds
+    formatting.dataset_repr(ds)
+
+    # indirect recursion
+    ds2 = xr.Dataset({"b": ("y", [1, 2, 3])})
+    ds.attrs["ds"] = ds2
+    ds2.attrs["ds"] = ds
+    formatting.dataset_repr(ds2)
+
+
 def test__element_formatter(n_elements: int = 100) -> None:
     expected = """\
     Dimensions without coordinates: dim_0: 3, dim_1: 3, dim_2: 3, dim_3: 3,
@@ -627,3 +709,18 @@ def test__element_formatter(n_elements: int = 100) -> None:
     )
     actual = intro + values
     assert expected == actual
+
+
+def test_lazy_array_wont_compute() -> None:
+    from xarray.core.indexing import LazilyIndexedArray
+
+    class LazilyIndexedArrayNotComputable(LazilyIndexedArray):
+        def __array__(self, dtype=None):
+            raise NotImplementedError("Computing this array is not possible.")
+
+    arr = LazilyIndexedArrayNotComputable(np.array([1, 2]))
+    var = xr.DataArray(arr)
+
+    # These will crash if var.data are converted to numpy arrays:
+    var.__repr__()
+    var._repr_html_()
