@@ -102,20 +102,20 @@ def test_repr_of_dataarray(dataarray) -> None:
     assert "dim_0" in formatted
     # has an expanded data section
     assert formatted.count("class='xr-array-in' type='checkbox' checked>") == 1
-    # coords and attrs don't have an items so they'll be be disabled and collapsed
+    # coords, indexes and attrs don't have an items so they'll be be disabled and collapsed
     assert (
-        formatted.count("class='xr-section-summary-in' type='checkbox' disabled >") == 2
+        formatted.count("class='xr-section-summary-in' type='checkbox' disabled >") == 3
     )
 
     with xr.set_options(display_expand_data=False):
         formatted = fh.array_repr(dataarray)
         assert "dim_0" in formatted
-        # has an expanded data section
+        # has a collapsed data section
         assert formatted.count("class='xr-array-in' type='checkbox' checked>") == 0
-        # coords and attrs don't have an items so they'll be be disabled and collapsed
+        # coords, indexes and attrs don't have an items so they'll be be disabled and collapsed
         assert (
             formatted.count("class='xr-section-summary-in' type='checkbox' disabled >")
-            == 2
+            == 3
         )
 
 
@@ -130,6 +130,8 @@ def test_repr_of_dataset(dataset) -> None:
     assert (
         formatted.count("class='xr-section-summary-in' type='checkbox'  checked>") == 3
     )
+    # indexes is collapsed
+    assert formatted.count("class='xr-section-summary-in' type='checkbox'  >") == 1
     assert "&lt;U4" in formatted or "&gt;U4" in formatted
     assert "&lt;IA&gt;" in formatted
 
@@ -137,12 +139,13 @@ def test_repr_of_dataset(dataset) -> None:
         display_expand_coords=False,
         display_expand_data_vars=False,
         display_expand_attrs=False,
+        display_expand_indexes=True,
     ):
         formatted = fh.dataset_repr(dataset)
-        # coords, attrs, and data_vars are collapsed
+        # coords, attrs, and data_vars are collapsed, indexes is expanded
         assert (
             formatted.count("class='xr-section-summary-in' type='checkbox'  checked>")
-            == 0
+            == 1
         )
         assert "&lt;U4" in formatted or "&gt;U4" in formatted
         assert "&lt;IA&gt;" in formatted
