@@ -8,19 +8,19 @@ import pandas as pd
 import pytest
 
 import xarray as xr
+from xarray.coding.cftimeindex import _parse_array_of_cftime_strings
 from xarray.core.types import InterpOptions
 from xarray.tests import (
     assert_allclose,
     assert_equal,
     assert_identical,
+    has_dask,
+    has_scipy,
     requires_cftime,
     requires_dask,
     requires_scipy,
 )
-
-from ..coding.cftimeindex import _parse_array_of_cftime_strings
-from . import has_dask, has_scipy
-from .test_dataset import create_test_data
+from xarray.tests.test_dataset import create_test_data
 
 try:
     import scipy
