@@ -3,8 +3,7 @@ from __future__ import annotations
 import pytest
 
 from xarray import DataArray, tutorial
-
-from . import assert_identical, network
+from xarray.tests import assert_identical, network
 
 
 @network
