@@ -4,10 +4,9 @@ import numpy as np
 import pytest
 
 import xarray as xr
-
-from . import assert_array_equal
-from . import assert_identical as assert_identical_
-from . import mock
+from xarray.tests import assert_array_equal
+from xarray.tests import assert_identical as assert_identical_
+from xarray.tests import mock
 
 
 def assert_identical(a, b):
